
'--------------------------------------------------------------------
        'Puzzle Translator Program
'--------------------------------------------------------------------

        'Author - Tim Harvey
        'Date   - March 2002
        'Written in Quick Basic 4.5

'This program reads the contents of a compiled crossword puzzle in
'the Across Lite format (www.litsoft.com) and translates these into
'a pzl format which can be converted by an existing GEOS app into
'a crossword puzzle which can be opened and used in the current
'New Deal crossword program.

'P.S. Grateful thanks to Gene Anderson (www.designsinlight.com) who worked
'out an odd issue with the GEOS crossword converter.

'--------------------------------------------------------------------



DECLARE FUNCTION GetAClue$ ()

GetInfilename:
        LINE INPUT "The file name is "; fi$
        fi$ = UCASE$(fi$)
        IF fi$ = "HELP" THEN SHELL "dir *.puz /p": SHELL "pause": GOTO GetInfilename
        IF fi$ = "" GOTO GetInfilename
        IF fi$ = "END" THEN END


        'Initialize a few variables

CONST NO = 0, YES = NOT NO

        'Open the file as binary so we can read non-ASCII characters

OPEN fi$ FOR BINARY AS #1

        'Set these as a single space so that the GET #1 only reads in
        'a single byte into the variables.

CwdWidth$ = " ": CwdHeight$ = " "

GET #1, 45, CwdWidth$
GET #1, 46, CwdHeight$

CwdWidth% = ASC(CwdWidth$): CwdHeight% = ASC(CwdHeight$)
PuzzleSize% = CwdWidth% * CwdHeight%

        'Now we know the size of the puzzle we are dealing with, initialize
        'the arrays that will hold the parts of the puzzle.

DIM HeaderInfo$(7)
DIM CrosswordSolution$(CwdHeight% + 1)
DIM AcrossClue$(PuzzleSize% \ 2)
DIM DownClue$(PuzzleSize% \ 2)

        'this was used just to confirm the program was working.
        'It's still kind of fun to see in the output to the screen.

'PRINT "x = "; CwdWidth%; "   y = "; CwdHeight%; "Puzzle size = "; PuzzleSize%


        'This sets CrossWord$ to be a blank string the byte-length of
        'the puzzle and pulls it all in at once.

Crossword$ = STRING$(PuzzleSize%, " ")

GET #1, 53, Crossword$

        'This pulls in each line of the puzzle, sends one copy to the screen
        'and another copy to a string array.

FOR i% = 0 TO CwdHeight% - 1
LineOfPuzzle$ = MID$(Crossword$, CwdWidth% * i% + 1, CwdWidth%)
'PRINT LineOfPuzzle$
CrosswordSolution$(i% + 1) = LineOfPuzzle$
NEXT i%

        'this puts the terminating stars at the end of the block, which are
        'needed for the GEOS crossword translator.

CrosswordSolution$(i% + 1) = "***"

        'This bit reads in the solution area then discards it, to align the
        'GET pointer at the start of the crossword info (titles, credits etc).

ignore$ = Crossword$
GET #1, , ignore$


        'The Title, Author & Copyright bits precede the clues, so
        'put them into variables and then proceed to the rest of the clues.

Title$ = GetAClue$
Author$ = GetAClue$
Copyr$ = GetAClue$

        'Initialise a few more variables we will need in a moment.

ThisSq% = 0
CluNo% = 1
DownCounter% = 0
AcrossCounter% = 0
               
        'This is where most of the work gets done.

NumberSquares:

        'This goes through the squares one-by-one checking to see if
        'they are the start of words or not.

        IF ThisSq% < PuzzleSize% THEN
                ThisSq% = ThisSq% + 1
        ELSE GOTO FinishUp
        END IF

        'this sets the default values of each square as "NO"

ACROSS = NO
DOWN = NO
Temp$ = ""

        'this is to allow us to locate each letter by row-column co-ordinate.

Column% = ThisSq% MOD CwdWidth%

Row% = 0
DO
Row% = Row% + 1
LOOP UNTIL ThisSq% <= (Row% * CwdWidth%)


        'this allows us to determine what's in the current square.
       
ThisSq$ = MID$(Crossword$, ThisSq%, 1)

        'If the current square is on any edge of the puzzle, then set it as
        'being notionally next to a blank (ie a dot).
        'This simplifies the decision of whether it is a square where a word
        'starts or not, by just comparing neighbouring squares.
       
    IF Column% = 1 THEN PrevSq$ = "." ELSE PrevSq$ = MID$(Crossword$, ThisSq% - 1, 1)
    IF Column% = 0 THEN NextSq$ = "." ELSE NextSq$ = MID$(Crossword$, ThisSq% + 1, 1)

    IF Row% = 1 THEN UpASq$ = "." ELSE UpASq$ = MID$(Crossword$, ThisSq% - CwdWidth%, 1)
    IF Row% = CwdHeight% THEN DwnASq$ = "." ELSE DwnASq$ = MID$(Crossword$, ThisSq% + CwdWidth%, 1)

        'The checks here are drop-through. These determine whether
        'the square is the start of a word across or down or both or neither.
        'NO is the default state and the last state to be set.

    IF PrevSq$ = "." AND NextSq$ <> "." THEN ACROSS = YES
    IF PrevSq$ <> "." OR NextSq$ = "." OR ThisSq$ = "." THEN ACROSS = NO

    IF UpASq$ = "." AND DwnASq$ <> "." THEN DOWN = YES
    IF UpASq$ <> "." OR DwnASq$ = "." OR ThisSq$ = "." THEN DOWN = NO

        'If a word starts in this square, increment the Clue counter.

    IF ACROSS = YES OR DOWN = YES THEN ClueNo% = ClueNo% + 1
      

AssignClues:
        
        IF ACROSS = NO AND DOWN = YES THEN
                Temp$ = LTRIM$(STR$(ClueNo%)) + " " + GetAClue$
                DownCounter% = DownCounter% + 1
                DownClue$(DownCounter%) = Temp$
        END IF
       
        IF ACROSS = YES AND DOWN = NO THEN
                Temp$ = LTRIM$(STR$(ClueNo%)) + " " + GetAClue$
                AcrossCounter% = AcrossCounter% + 1
                AcrossClue$(AcrossCounter%) = Temp$
        END IF

        IF ACROSS = YES AND DOWN = YES THEN
                Temp$ = LTRIM$(STR$(ClueNo%)) + " " + GetAClue$
                AcrossCounter% = AcrossCounter% + 1
                AcrossClue$(AcrossCounter%) = Temp$
                Temp$ = LTRIM$(STR$(ClueNo%)) + " " + GetAClue$
                DownCounter% = DownCounter% + 1
                DownClue$(DownCounter%) = Temp$
        END IF

       
        GOTO NumberSquares


FinishUp:

        'this sets the output file to have the same name as the
        'input file, but with a new pzl extension which the New Deal
        'puzzle converter recognizes by default.

fo$ = MID$(fi$, 1, LEN(fi$) - 4) + ".pzl"

        'Assign HeaderInfo$ information.

HeaderInfo$(1) = "CODEPAGE=437"
HeaderInfo$(2) = "AUTHOR=" + Author$
       
        'All new files are given a "Medium" rating for New Deal, as
        'Across Lite crosswords do not have a difficulty rating to transfer.
        
HeaderInfo$(3) = "RATING=5"

HeaderInfo$(4) = "TITLE=" + Title$
HeaderInfo$(5) = "COPYRIGHT=" + Copyr$
HeaderInfo$(6) = "SOURCE=See Above"
HeaderInfo$(7) = "***"

        'Open the output file and immediately close it.
        'This zeros out anything that's in there already.

OPEN fo$ FOR OUTPUT AS #2
CLOSE #2

        'Now we send all the data out in order.

OPEN fo$ FOR APPEND AS #2

        'Write the required information to the output file, in order.

        'Send out the Header information, plus terminator asterisks.

FOR i% = 1 TO 7
PRINT #2, HeaderInfo$(i%)
NEXT i%

        'Send out the Crossword itself, plus terminator asterisks.

FOR i% = 1 TO CwdHeight% + 1
PRINT #2, CrosswordSolution$(i%)
NEXT i%

        'Send out Across clues, plus terminator asterisks.

FOR i% = 1 TO AcrossCounter%
PRINT #2, AcrossClue$(i%)
NEXT i%

PRINT #2, "***"

        'Send out Down clues, plus terminator asterisks.

FOR i% = 1 TO DownCounter%
PRINT #2, DownClue$(i%)
NEXT i%

PRINT #2, "***"

        'That's all folks! Beep!

CLOSE : BEEP: END

FUNCTION GetAClue$
'-----------------------------------------------------------------
        'This function recovers text strings between hex(00)
        'which Across Lite crosswords use for markers.
        'Function reads a bytes at a time, but only adds
        'them to the clues if they are alphanumeric, or
        'certain other punctuation marks.
'-----------------------------------------------------------------


        'Zero out Clue$. Set CluBit$ to a single space as this allows
        'it to read in a single byte at a time.

Clue$ = ""
CluBit$ = " "
 
StartHere:

     IF NOT EOF(1) THEN GET #1, , CluBit$ ELSE BEEP: END
     
     'The crossword translator has problems with non-alphanumeric,
     'particularly colons, so filter out anything else.

     'Colons can be part valid formatting, so replace these with semi-colons.
    
     IF CluBit$ = ":" THEN CluBit$ = ";"

     'This looks messy, but you can see everything on one screen and
     'it's really easy to remove or add characters if you need to.

     IF INSTR("abcdefghijklmnopqrstuvwxyz", CluBit$) THEN Clue$ = Clue$ + CluBit$: GOTO StartHere
     IF INSTR("ABCDEFGHIJKLMNOPQRSTUVWXYZ", CluBit$) THEN Clue$ = Clue$ + CluBit$: GOTO StartHere
     IF INSTR("0123456789", CluBit$) THEN Clue$ = Clue$ + CluBit$: GOTO StartHere
     IF INSTR(";'~` !@#$%^&*()-_+=[]{}|<>,./?", CluBit$) THEN Clue$ = Clue$ + CluBit$: GOTO StartHere
    
     'A hex(00), marks the end of a clue or piece of
     'text, so stop looking when you find one and send
     'out the text you've got. Otherwise, loop around
     'and look for the next character.

     IF ASC(CluBit$) <> 0 THEN GOTO StartHere


     GetAClue$ = Clue$

END FUNCTION

